import Mocker from './mocker/mocker';
export default class Index {
    private static isEnabled;
    /**
     * Auto detect request environment and setup request mock for wx.request, fetch and XHR.
     * @param {string} proxyServer A proxy server which is used by proxy mode.
     */
    static setup(proxyServer?: string): Mocker;
    /**
     * Setup request mock for wx.request.
     * @param {string} proxyServer A proxy server which is used by proxy mode.
     */
    static setupForWx(proxyServer?: string): Mocker;
    /**
     * Setup request mock for XMLHttpRequest.
     * @param {string} proxyServer A proxy server which is used by proxy mode.
     */
    static setupForXhr(proxyServer?: string): Mocker;
    /**
     * Setup request mock for fetch.
     * @param {string} proxyServer A proxy server which is used by proxy mode.
     */
    static setupForFetch(proxyServer?: string): Mocker;
    /**
     * Setup request mock for node http/https request.
     * For http.get, https.get, http.request, https.request in nodejs environment
     * @param {string} proxyServer A proxy server which is used by proxy mode.
     */
    static setupForNode(proxyServer?: string): Mocker;
    /**
     * Enable mock function temporarily.
     * Not available in proxy mode.
     */
    static enable(): Mocker;
    /**
     * Disable mock function temporarily.
     * Not available in proxy mode.
     */
    static disable(): Mocker;
    /**
     * Enable verbose log.
     * Not available in proxy mode.
     */
    static enableLog(): Mocker;
    /**
     * Disable verbose log.
     * Not available in proxy mode.
     */
    static disableLog(): Mocker;
    /**
     * Setup request mock for unit test.
     * @param {string} type
     */
    static setupForUnitTest(type: 'wx' | 'xhr' | 'fetch' | 'node' | 'all'): Mocker;
    static default: typeof Index;
}
