"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
var utils_1 = require("./common/utils");
var fetch_1 = __importDefault(require("./interceptor/fetch"));
var wx_request_1 = __importDefault(require("./interceptor/wx-request"));
var xml_http_request_1 = __importDefault(require("./interceptor/xml-http-request"));
var mocker_1 = __importDefault(require("./mocker/mocker"));
var Index = /** @class */ (function () {
    function Index() {
    }
    /**
     * Auto detect request environment and setup request mock for wx.request, fetch and XHR.
     * @param {string} proxyServer A proxy server which is used by proxy mode.
     */
    Index.setup = function (proxyServer) {
        if (proxyServer === void 0) { proxyServer = ''; }
        var mocker = new mocker_1.default(proxyServer);
        if (this.isEnabled && typeof wx !== 'undefined' && typeof wx.request === 'function') {
            wx_request_1.default.setup(mocker, proxyServer);
        }
        if (this.isEnabled && typeof window !== 'undefined' && typeof window.XMLHttpRequest === 'function') {
            xml_http_request_1.default.setup(mocker, proxyServer);
        }
        if (this.isEnabled && typeof window !== 'undefined' && typeof window.fetch === 'function') {
            fetch_1.default.setup(mocker, proxyServer);
        }
        // for http.get, https.get, http.request, https.request in node environment
        if (this.isEnabled && (0, utils_1.isNodejs)()) {
            // use require here to avoid static analysis
            // eslint-disable-next-line @typescript-eslint/no-var-requires
            require('./interceptor/node/http-and-https').default.setup(mocker, proxyServer);
        }
        return mocker;
    };
    /**
     * Setup request mock for wx.request.
     * @param {string} proxyServer A proxy server which is used by proxy mode.
     */
    Index.setupForWx = function (proxyServer) {
        if (proxyServer === void 0) { proxyServer = ''; }
        var mocker = new mocker_1.default(proxyServer);
        this.isEnabled && wx_request_1.default.setup(mocker, proxyServer);
        return mocker;
    };
    /**
     * Setup request mock for XMLHttpRequest.
     * @param {string} proxyServer A proxy server which is used by proxy mode.
     */
    Index.setupForXhr = function (proxyServer) {
        if (proxyServer === void 0) { proxyServer = ''; }
        var mocker = new mocker_1.default(proxyServer);
        this.isEnabled && xml_http_request_1.default.setup(mocker, proxyServer);
        return mocker;
    };
    /**
     * Setup request mock for fetch.
     * @param {string} proxyServer A proxy server which is used by proxy mode.
     */
    Index.setupForFetch = function (proxyServer) {
        if (proxyServer === void 0) { proxyServer = ''; }
        var mocker = new mocker_1.default(proxyServer);
        this.isEnabled && fetch_1.default.setup(mocker, proxyServer);
        return mocker;
    };
    /**
     * Setup request mock for node http/https request.
     * For http.get, https.get, http.request, https.request in nodejs environment
     * @param {string} proxyServer A proxy server which is used by proxy mode.
     */
    Index.setupForNode = function (proxyServer) {
        if (proxyServer === void 0) { proxyServer = ''; }
        var mocker = new mocker_1.default(proxyServer);
        // use require here to avoid static analysis
        // eslint-disable-next-line @typescript-eslint/no-var-requires
        this.isEnabled && require('./interceptor/node/http-and-https').default.setup(mocker, proxyServer);
        return mocker;
    };
    /**
     * Enable mock function temporarily.
     * Not available in proxy mode.
     */
    Index.enable = function () {
        this.isEnabled = true;
        return mocker_1.default.getInstance().enable();
    };
    /**
     * Disable mock function temporarily.
     * Not available in proxy mode.
     */
    Index.disable = function () {
        this.isEnabled = false;
        return mocker_1.default.getInstance().disable();
    };
    /**
     * Enable verbose log.
     * Not available in proxy mode.
     */
    Index.enableLog = function () {
        return mocker_1.default.getInstance().enableLog();
    };
    /**
     * Disable verbose log.
     * Not available in proxy mode.
     */
    Index.disableLog = function () {
        return mocker_1.default.getInstance().disableLog();
    };
    /**
     * Setup request mock for unit test.
     * @param {string} type
     */
    Index.setupForUnitTest = function (type) {
        if (!(0, utils_1.isNodejs)()) {
            throw new Error('"setupForUnitTest" is only for nodejs envrioment.');
        }
        if (!['wx', 'xhr', 'fetch', 'node', 'all'].includes(type)) {
            throw new Error('Invalid type, valid types are "wx", "xhr", "fetch", "node" and "all".');
        }
        var mocker = new mocker_1.default();
        if (type === 'wx' || type === 'all') {
            wx_request_1.default.initDummyWxRequestForUnitTest(mocker);
        }
        if (type === 'xhr' || type === 'all') {
            xml_http_request_1.default.initDummyXHRForUnitTest(mocker);
        }
        if (type === 'fetch' || type === 'all') {
            fetch_1.default.initDummyFetchForUnitTest(mocker);
        }
        if (type === 'node' || type === 'all') {
            // use require here to avoid static analysis
            // eslint-disable-next-line @typescript-eslint/no-var-requires
            require('./interceptor/node/http-and-https').default.setup(mocker);
        }
        return mocker;
    };
    Index.isEnabled = true;
    Index.default = Index; // for backward compatibility
    return Index;
}());
exports.default = Index;
module.exports = Index; // without 'default' property
//# sourceMappingURL=index.js.map